<?php

namespace Drupal\media_fotoweb;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides common helper for building the Fotoweb widget.
 */
trait FotowebWidgetTrait {

  /**
   * Builds the Fotoweb Embedded Selection Widget as an <iframe>.
   *
   * @return string
   *   Returns the embedded widget markup.
   */
  protected function buildEmbeddedSelectionWidget() {
    $config = $this->configFactory->get('media_fotoweb.settings');
    $fotoweb_host = $config->get('server');
    $widget_url = $fotoweb_host . '/fotoweb/widgets/selection';
    $widget_height = $config->get('selection_widget_height');

    // Generate user login token and append it to widget url, when using SSO.
    if ($config->get('selection_widget_use_sso')) {
      if ($user_login_token = $this->loginManager->getLoginTokenFromAccount($this->currentUser)) {
        $widget_url .= '?lt=' . $user_login_token;
      }
    }

    // Create the widget using an <iframe>.
    $build = new FormattableMarkup('<iframe src=":widget_url" width="100%" height=":widget_height"></iframe>', [
      ':widget_url' => $widget_url,
      ':widget_height' => $widget_height,
    ]);

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function prepareEntities(array $form, FormStateInterface   $form_state) {
    $entity_type_id = 'fotoweb';
    $entities = [];
    $selected = json_decode($form_state->getValue('fotoweb_selected', []));

    /** @var \Drupal\media\MediaTypeInterface $media_type */
    $media_type = $this->entityTypeManager->getStorage('media_type')
      ->load($entity_type_id);
    /** @var \Drupal\media_fotoweb\Plugin\media\Source\Fotoweb $plugin */
    $plugin = $media_type->getSource();
    $source_field = $plugin->getConfiguration()['source_field'];

    if (!empty($selected)) {
      foreach ($selected as $asset) {
        $mid = $this->entityTypeManager->getStorage('media')->getQuery()
          ->condition($source_field, $asset->href)
          ->range(0, 1)
          ->execute();
        if ($mid) {
          $media = $this->loadAndSyncMedia($mid, $asset);
          $entities[] = $media;
        }
        else {
          /** @var \Drupal\media\MediaInterface $media */
          $media = $this->entityTypeManager->getStorage('media')->create([
            'bundle' => $media_type->id(),
            $source_field => $asset->href,
            'uid' => $this->currentUser->id(),
            'status' => TRUE,
            'original_data' => $asset,
          ]);

          $media->save();
          $entities[] = $media;
        }
      }
    }

    return $entities;
  }

  /**
   * Load and sync the existing media entity.
   *
   * @param int $mid
   *   The media id.
   * @param object $asset
   *   The Fotoweb asset.
   *
   * @return \Drupal\media\MediaInterface
   *   The selected media entity.
   */
  protected function loadAndSyncMedia($mid, object $asset) {
    $config = $this->configFactory->get('media_fotoweb.settings');
    $asset_update_type = $config->get('asset_update_type');

    /** @var \Drupal\media\MediaInterface $media */
    $media = $this->entityTypeManager->getStorage('media')
      ->load(reset($mid));

    // Resync the asset data from Fotoweb on selection, when "reuse" update
    // type was set.
    if ($asset_update_type === 'reused') {
      $media->original_data = $asset;
      $media->save();
    }

    return $media;
  }

}